#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Follow Me Helix
#      By:	sdmitch
#   Usage:	creates a helix around a selected centerline.  
#    Note:	This is a disk and resource hog so be very careful not to make the CL to long!	
#						Test on a short centerline to verify input parameters.
#    Date:	Sep 2011
#------------------------------------------------------------------------------------------------
require 'sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Helix') { SDM_HM.helix_maker }
end
#
file_loaded(filename)
#
module SDM_HM

	def self.helix_maker
		sel=Sketchup.active_model.selection
		if (sel.length == 0) then
			UI.messagebox "Select the lines and/or curves that will be the shape and length of the helix you want to create"
			return
		else
			ans=UI.inputbox(["Radius of helix:","linear Increment:","Angle Increment:"],[1.inch,0.125.inch,15],"Helix Generator")
			radius=ans[0].to_l;length_increment = ans[1].to_l; angle_increment=ans[2].to_i
			verts = self.sort_edges(sel,2); last = verts.length - 1
			Sketchup.active_model.start_operation("Helix")
			start_el = 0; starting_angle = 0; pts = []
			1.upto(last) do |i|
					pt0=verts[i-1]
					vector=verts[i-1].vector_to(verts[i])
					length=verts[i-1].distance(verts[i])
					xaxis,yaxis,zaxis=vector.axes; el = start_el
					angle = starting_angle; xaxis.length = radius; yaxis.length = radius
					while el < length
						cosa = Math.cos(angle.degrees); sina = Math.sin(angle.degrees)
						vec = Geom::Vector3d.linear_combination(cosa,xaxis,sina,yaxis)
						pts.push pt0.offset(vector,el)+vec
						el += length_increment;
						angle += angle_increment; angle = 0 if angle >= 360
					end
					starting_angle = angle; start_el = el - length
			end
			pt0 = pts[0]; vector = pts[0].vector_to(pts[1])
			curve=Sketchup.active_model.entities.add_curve(pts)
			Sketchup.active_model.commit_operation
		end
	end

	def self.sort_edges(sel,flag)
		edges=sel.select {|x| x.is_a? Sketchup::Edge}
	#	---------------- This code copied from FAK.rb ------------------
		return if edges.length==0
		re=[];rp=[]	# split the output into two arrays
		while edges.length!=0
			start=nil
			edges.each{|x|
			start=x.start; break  if (x.start.edges&edges).length==1
			start=x.end; break if (x.end.edges&edges).length==1
			}
			vts=[]; curve=[]
			while start
				next_edge=(start.edges&edges-curve)[0]
				if next_edge
					curve << next_edge
					vts << start
					start=next_edge.other_vertex(start)
				else
					vts << start
					start=nil
				end
			end
			edges-=curve
			re += curve 
			rp += vts 
		end
		rp.collect!{|x| x.position}
		return rp if flag==2
		return re if flag==1
		return re,rp	#re=edges, rp=points
	end
	
end
